searchData={"items":[{"type":"module","title":"Nebulex.Distributed","doc":"Distributed caching functionality for Nebulex.\n\nThis project provides the core functionality for distributed caching in\nNebulex, primarily through the following adapter components:\n\n  * `Nebulex.Adapters.Partitioned` - The Partitioned adapter implements a\n    distributed cache where data is partitioned across multiple nodes in a\n    cluster. Each node is responsible for a subset of the cache data.\n  * `Nebulex.Adapters.Multilevel` - The Multilevel adapter implements a cache\n    hierarchy with multiple levels, where each level can use a different cache\n    adapter. This is particularly useful for implementing near-cache\n    topologies.\n  * `Nebulex.Adapters.Replicated` - The Replicated adapter implements a\n    distributed cache where data is replicated across all nodes in a cluster.\n    Each node has an identical copy of the cache data. **WIP**.\n\nCheck the adapters documentation for more information on how to configure and\nuse them.","ref":"Nebulex.Distributed.html"},{"type":"module","title":"Nebulex.Adapters.Multilevel","doc":"Adapter module for the multi-level cache topogy.\n\nThe Multi-level adapter is a simple layer that works on top of a local or\ndistributed cache implementation, enabling a cache hierarchy by levels.\nMulti-level caches generally operate by checking the fastest,\nlevel 1 (L1) cache first; if it hits, the adapter proceeds at\nhigh speed. If that first cache misses, the next fastest cache\n(level 2, L2) is checked, and so on, before accessing external\nmemory (that can be handled by a `cacheable` decorator).\n\nFor write functions, the \"Write Through\" policy is applied by default;\nthis policy ensures that the data is stored safely as it is written\nthroughout the hierarchy. However, it is possible to force the write\noperation in a specific level (although it is not recommended) via\n`level` option, where the value is a positive integer greater than 0.\n\nWe can define a multi-level cache as follows:\n\n    defmodule MyApp.Multilevel do\n      use Nebulex.Cache,\n        otp_app: :nebulex,\n        adapter: Nebulex.Adapters.Multilevel\n\n      defmodule L1 do\n        use Nebulex.Cache,\n          otp_app: :nebulex,\n          adapter: Nebulex.Adapters.Local\n      end\n\n      defmodule L2 do\n        use Nebulex.Cache,\n          otp_app: :nebulex,\n          adapter: Nebulex.Adapters.Partitioned\n      end\n    end\n\nWhere the configuration for the cache and its levels must be in your\napplication environment, usually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.Multilevel,\n      inclusion_policy: :inclusive,\n      levels: [\n        {\n          MyApp.Multilevel.L1,\n          gc_interval: :timer.hours(12),\n          backend: :shards\n        },\n        {\n          MyApp.Multilevel.L2,\n          primary: [\n            gc_interval: :timer.hours(12),\n            backend: :shards\n          ]\n        }\n      ]\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.Multilevel, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Multilevel.html"},{"type":"module","title":"Options - Nebulex.Adapters.Multilevel","doc":"This adapter supports the following options and all of them can be given via\nthe cache configuration:\n\n* `:stats` (`t:boolean/0`) - A flag to determine whether to collect cache stats. The default value is `true`.\n\n* `:levels` (non-empty `t:keyword/0`) - Required. This option is to define the levels, a list of tuples in the shape\n  `{cache_level :: Nebulex.Cache.t(), opts :: keyword()}`, where the first\n  element is the module that defines the cache for that level, and the\n  second one is the options given to that level in the `start_link/1`.\n  The multi-level cache adapter relies on the list order to determine the\n  level hierarchy. For example, the first element in the list will be the\n  L1 cache (level 1), and so on; the Nth element will be the LN cache.\n  This option is required; if it is not set or empty, the adapter raises\n  an exception.\n\n* `:inclusion_policy` - Specifies the cache inclusion policy: `:inclusive` or `:exclusive`.\n\n  For an \"inclusive\" cache, the same data can be present in all cache\n  levels. On the other hand, in an \"exclusive\" cache, the data can be\n  present in only one cache level; the key cannot exist in the rest of the\n  levels at the same time. This option applies to the callback `get` only;\n  if the cache inclusion policy is `:inclusive`, when the key does exist in a level\n  N, that entry is duplicated backward (to all previous levels: 1..N-1).\n  However, when the mode is `:inclusive`, the `get_all` operation is\n  translated into multiple `get` calls underneath (which may be a\n  significant performance penalty) since it requires replicating the entries\n  properly with their current TTLs. It is possible to skip the replication\n  when calling `get_all` using the option `:replicate`.\n\n  The default value is `:inclusive`.","ref":"Nebulex.Adapters.Multilevel.html#module-options"},{"type":"module","title":"Shared options - Nebulex.Adapters.Multilevel","doc":"Almost all of the cache functions outlined in `Nebulex.Cache` module\naccept the following options:\n\n* `:timeout` (`t:timeout/0`) - The time in **milliseconds** to wait for a command to finish\n  (`:infinity` to wait indefinitely). The default value is `5000`.\n\n* `:level` (`t:pos_integer/0`) - Dictates the level where the cache command will take place. The evaluation\n  is performed by default throughout the cache hierarchy (all levels).","ref":"Nebulex.Adapters.Multilevel.html#module-shared-options"},{"type":"module","title":"Queryable API options - Nebulex.Adapters.Multilevel","doc":"The following options apply to `get_all`, `count_all`, `delete_all`,\nand `stream` commands:\n\n* `:replicate` (`t:boolean/0`) - This option applies only to the `get_all` callback when using the\n  inclusive policy. Determines whether the entries should be replicated\n  to the backward levels or not. The default value is `true`.\n\n* `:on_error` (`:raise` | `:nothing`) - Indicates whether to raise an exception when an error occurs or do nothing\n  (skip errors).\n\n  When the stream is evaluated, the adapter attempts to execute the `stream`\n  command on the different cache levels. Still, the execution could fail at\n  any of the cache levels. If the option is set to `:raise`, the command\n  will raise an exception when an error occurs on the stream evaluation.\n  On the other hand, if it is set to `:nothing`, the error is skipped.\n\n  The default value is `:raise`.","ref":"Nebulex.Adapters.Multilevel.html#module-queryable-api-options"},{"type":"module","title":"Telemetry events - Nebulex.Adapters.Multilevel","doc":"Since the multi-level adapter works as a wrapper for the configured cache\nlevels, these will emit the Nebulex Telemetry events. Therefore, there will\nbe events emitted for each cache level. For example, the cache defined before\n`MyApp.Multilevel` will emit the following events:\n\n  * Top level cache:\n    * `[:my_app, :multilevel, :command, :start]`\n    * `[:my_app, :multilevel, :command, :stop]`\n    * `[:my_app, :multilevel, :command, :exception]`\n\n  * L1 cache:\n    * `[:my_app, :multilevel, :command, :start]`\n    * `[:my_app, :multilevel, :command, :stop]`\n    * `[:my_app, :multilevel, :command, :exception]`\n\n  * L2 cache:\n    * `[:my_app, :multilevel, :command, :start]`\n    * `[:my_app, :multilevel, :primary, :command, :start]`\n    * `[:my_app, :multilevel, :command, :stop]`\n    * `[:my_app, :multilevel, :primary, :command, :stop]`\n    * `[:my_app, :multilevel, :command, :exception]`\n    * `[:my_app, :multilevel, :primary, :command, :exception]`\n\nAs you may notice, the telemetry prefix by default for all cache levels is\n`[:my_app, :multilevel]`, but you can get the details about the cache from\nthe metadata. Alternatively, you could specify the `:telemetry_prefix` for\neach cache level within the `:levels` option. For example:\n\n    config :my_app, MyApp.Multilevel,\n      levels: [\n        {MyApp.Multilevel.L1, telemetry_prefix: [:my_app, :multilevel, :l1]},\n        {MyApp.Multilevel.L2, telemetry_prefix: [:my_app, :multilevel, :l2]}\n      ]\n\nIn this case, the telemetry prefix for the L1 cache will be\n`[:my_app, :multilevel, :l1]` and the L2 cache will be\n`[:my_app, :multilevel, :l2]`.\n\nSee also the [Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html)\nfor more information.","ref":"Nebulex.Adapters.Multilevel.html#module-telemetry-events"},{"type":"module","title":"Info API - Nebulex.Adapters.Multilevel","doc":"As explained above, the multi-level adapter uses the configured cache levels.\nTherefore, the information provided by the `info` command will depend on the\nadapters configured for each level. The Nebulex built-in adapters support the\nrecommended keys `:server`, `:memory`, and `:stats`. Additionally, the\nmulti-level adapter supports:\n\n  * `:levels_info` - A list with the info map for each cache level.\n\nFor example, the info for `MyApp.Multilevel` may look like this:\n\n    iex> MyApp.Multilevel.info!()\n    %{\n      memory: %{total: nil, used: 206760},\n      server: %{\n        cache_module: MyApp.Multilevel,\n        cache_name: :multilevel_inclusive,\n        cache_adapter: Nebulex.Adapters.Multilevel,\n        cache_pid: #PID<0.998.0>,\n        nbx_version: \"3.0.0\"\n      },\n      stats: %{\n        hits: 0,\n        misses: 0,\n        writes: 0,\n        evictions: 0,\n        expirations: 0,\n        deletions: 0,\n        updates: 0\n      },\n      levels_info: [\n        %{\n          memory: %{total: nil, used: 68920},\n          server: %{\n            cache_module: MyApp.Multilevel.L1,\n            cache_name: MyApp.Multilevel.L1,\n            cache_adapter: Nebulex.Adapters.Local,\n            cache_pid: #PID<0.1000.0>,\n            nbx_version: \"3.0.0\"\n          },\n          stats: %{\n            hits: 0,\n            misses: 0,\n            writes: 0,\n            evictions: 0,\n            expirations: 0,\n            deletions: 0,\n            updates: 0\n          }\n        },\n        %{\n          memory: %{total: nil, used: 68920},\n          nodes: [:\"node1@127.0.0.1\"],\n          server: %{\n            cache_module: MyApp.Multilevel.L2,\n            cache_name: MyApp.Multilevel.L2,\n            cache_adapter: Nebulex.Adapters.Partitioned,\n            cache_pid: #PID<0.1015.0>,\n            nbx_version: \"3.0.0\"\n          },\n          stats: %{\n            hits: 0,\n            misses: 0,\n            writes: 0,\n            evictions: 0,\n            expirations: 0,\n            deletions: 0,\n            updates: 0\n          },\n          nodes_info: %{\n            \"node1@127.0.0.1\": %{\n              memory: %{total: nil, used: 68920},\n              server: %{\n                cache_module: MyApp.Multilevel.L2.Primary,\n                cache_name: MyApp.Multilevel.L2.Primary,\n                cache_adapter: Nebulex.Adapters.Local,\n                cache_pid: #PID<0.1017.0>,\n                nbx_version: \"3.0.0\"\n              },\n              stats: %{\n                hits: 0,\n                misses: 0,\n                writes: 0,\n                evictions: 0,\n                expirations: 0,\n                deletions: 0,\n                updates: 0\n              }\n            }\n          }\n        }\n      ]\n    }","ref":"Nebulex.Adapters.Multilevel.html#module-info-api"},{"type":"module","title":"Extended API - Nebulex.Adapters.Multilevel","doc":"This adapter provides some additional convenience functions to the\n`Nebulex.Cache` API.\n\n### `inclusion_policy/0,1`\n\nReturns the inclusion policy of the cache.\n\n    iex> MyCache.inclusion_policy()\n    :inclusive","ref":"Nebulex.Adapters.Multilevel.html#module-extended-api"},{"type":"module","title":"CAVEATS - Nebulex.Adapters.Multilevel","doc":"Because this adapter reuses other existing/configured adapters, it inherits\nall their limitations too. Therefore, it is highly recommended to check the\ndocumentation of the adapters to use.","ref":"Nebulex.Adapters.Multilevel.html#module-caveats"},{"type":"module","title":"Nebulex.Adapters.Partitioned","doc":"Adapter module for the partitioned cache topology.","ref":"Nebulex.Adapters.Partitioned.html"},{"type":"module","title":"Features - Nebulex.Adapters.Partitioned","doc":"* Partitioned cache topology (Sharding Distribution Model).\n  * `ExHashRing` for distributing the keys across the cluster members.\n  * Support for transactions via Erlang global name registration facility.\n  * Configurable primary storage adapter.","ref":"Nebulex.Adapters.Partitioned.html#module-features"},{"type":"module","title":"Partitioned Cache Topology - Nebulex.Adapters.Partitioned","doc":"There are several key points to consider about a partitioned cache:\n\n  * _**Partitioned**_: The data in a distributed cache is spread out over\n    all the servers in such a way that no two servers are responsible for\n    the same piece of cached data. This means that the size of the cache\n    and the processing power associated with the management of the cache\n    can grow linearly with the size of the cluster. Also, it means that\n    operations against data in the cache can be accomplished with a\n    \"single hop,\" in other words, involving at most one other server.\n\n  * _**Load-Balanced**_:  Since the data is spread out evenly over the\n    servers, the responsibility for managing the data is automatically\n    load-balanced across the cluster.\n\n  * _**Ownership**_: Exactly one node in the cluster is responsible for each\n    piece of data in the cache.\n\n  * _**Point-To-Point**_: The communication for the partitioned cache is all\n    point-to-point, enabling linear scalability.\n\n  * _**Location Transparency**_: Although the data is spread out across\n    cluster nodes, the exact same API is used to access the data, and the\n    same behavior is provided by each of the API methods. This is called\n    location transparency, which means that the developer does not have to\n    code based on the topology of the cache, since the API and its behavior\n    will be the same with a local cache, a replicated cache, or a distributed\n    cache.\n\n  * _**Failover**_: Failover of a distributed cache involves promoting backup\n    data to be primary storage. When a cluster node fails, all remaining\n    cluster nodes determine what data each holds in backup that the failed\n    cluster node had primary responsible for when it died. Those data becomes\n    the responsibility of whatever cluster node was the backup for the data.\n    However, this adapter does not provide fault-tolerance implementation,\n    each piece of data is kept in a single node/machine (via sharding), then,\n    if a node fails, the data kept by this node won't be available for the\n    rest of the cluster members.\n\n> Based on **\"Distributed Caching Essential Lessons\"** by **Cameron Purdy**\n  and [Coherence Partitioned Cache Service][oracle-pcs].\n\n[oracle-pcs]: https://docs.oracle.com/cd/E13924_01/coh.340/e13819/partitionedcacheservice.htm","ref":"Nebulex.Adapters.Partitioned.html#module-partitioned-cache-topology"},{"type":"module","title":"Additional implementation notes - Nebulex.Adapters.Partitioned","doc":"`:pg` is used under-the-hood by the adapter to manage the cluster nodes.\nWhen the partitioned cache is started in a node, it creates a group and joins\nit (the cache supervisor PID is joined to the group). Then, when a function\nis invoked, the adapter uses `ExHashRing` to determine which node should\nhandle the request based on the key's hash value. This ensures consistent\nkey distribution across the cluster nodes, even when nodes join or leave\nthe cluster.\n\nThe key distribution process works as follows:\n\n  1. Each node in the cluster is assigned a set of virtual nodes (vnodes) in\n    the hash ring.\n  2. When a key is accessed, `ExHashRing.Ring` is used to find the node\n    responsible for that key (the hash value is used to find the corresponding\n    vnode in the hash ring).\n  3. The request is routed to the physical node that owns that vnode.\n\nThis consistent hashing approach provides several benefits:\n\n  * Minimal key redistribution when nodes join or leave the cluster.\n  * Even distribution of keys across the cluster.\n  * Predictable key-to-node mapping.\n  * Efficient node lookup for key operations.\n\nWhen a partitioned cache supervisor dies (the cache is stopped or killed for some\nreason), the PID of that process is automatically removed from the PG group.\nThe hash ring is then automatically rebalanced to ensure keys are properly\ndistributed among the remaining nodes.\n\nThis adapter depends on a local cache adapter (primary storage), it adds\nan extra layer on top of it in order to distribute requests across a group\nof nodes, where is supposed the local cache is running already. However,\nyou don't need to define any additional cache module for the primary\nstorage, instead, the adapter initializes it automatically (it adds the\nprimary storage as part of the supervision tree) based on the given\n`:primary_storage_adapter` option.","ref":"Nebulex.Adapters.Partitioned.html#module-additional-implementation-notes"},{"type":"module","title":"Usage - Nebulex.Adapters.Partitioned","doc":"The cache expects the `:otp_app` and `:adapter` as options when used.\nThe `:otp_app` should point to an OTP application with the cache\nconfiguration. Optionally, you can configure the desired primary\nstorage adapter with the option `:primary_storage_adapter`\n(defaults to `Nebulex.Adapters.Local`). See the compile time options\nfor more information:\n\n* `:primary_storage_adapter` (`t:atom/0`) - The adapter for the primary storage. The default value is `Nebulex.Adapters.Local`.\n\n\n\nFor example:\n\n    defmodule MyApp.PartitionedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Partitioned\n    end\n\nProviding the `:primary_storage_adapter`:\n\n    defmodule MyApp.PartitionedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Partitioned,\n        primary_storage_adapter: Nebulex.Adapters.Local\n    end\n\nWhere the configuration for the cache must be in your application environment,\nusually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.PartitionedCache,\n      primary: [\n        gc_interval: 3_600_000,\n        backend: :shards\n      ]\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.PartitionedCache, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Partitioned.html#module-usage"},{"type":"module","title":"Configuration options - Nebulex.Adapters.Partitioned","doc":"This adapter supports the following configuration options:\n\n* `:primary` (`t:keyword/0`) - Options for the adapter configured via the `:primary_storage_adapter`\n  option. The options will vary depending on the adapter used. The default value is `[]`.\n\n* `:hash_ring` (`t:keyword/0`) - Options for the hash ring.\n  See [`ExHashRing.Ring.start_link/2`][ex_hash_ring] for more information.\n\n  [ex_hash_ring]: https://hexdocs.pm/ex_hash_ring/ExHashRing.Ring.html#start_link/1\n\n  The default value is `[]`.","ref":"Nebulex.Adapters.Partitioned.html#module-configuration-options"},{"type":"module","title":"Shared runtime options - Nebulex.Adapters.Partitioned","doc":"When using the partitioned adapter, all of the cache functions outlined in\n`Nebulex.Cache` accept the following options:\n\n* `:timeout` (`t:timeout/0`) - The time in **milliseconds** to wait for a command to finish\n  (`:infinity` to wait indefinitely). The default value is `5000`.","ref":"Nebulex.Adapters.Partitioned.html#module-shared-runtime-options"},{"type":"module","title":"Stream options - Nebulex.Adapters.Partitioned","doc":"The `stream` command supports the following options:\n\n* `:on_error` (`:raise` | `:nothing`) - Indicates whether to raise an exception when an error occurs or do nothing\n  (skip errors).\n\n  When the stream is evaluated, the adapter attempts to execute the `stream`\n  command on the different nodes. Still, the execution could fail due to an\n  RPC error or the command explicitly returns an error. If the option is set\n  to `:raise`, the command will raise an exception when an error occurs on\n  the stream evaluation. On the other hand, if it is set to `:nothing`, the\n  error is skipped.\n\n  The default value is `:raise`.","ref":"Nebulex.Adapters.Partitioned.html#module-stream-options"},{"type":"module","title":"Telemetry events - Nebulex.Adapters.Partitioned","doc":"Since the partitioned adapter depends on the configured primary storage\ncache (which uses a local cache adapter), this one will also emit Telemetry\nevents. Therefore, there will be events emitted by the partitioned adapter\nas well as the primary storage cache. For example, the cache defined before\n`MyApp.PartitionedCache` will emit the following events:\n\n  * `[:my_app, :partitioned_cache, :command, :start]`\n  * `[:my_app, :partitioned_cache, :primary, :command, :start]`\n  * `[:my_app, :partitioned_cache, :command, :stop]`\n  * `[:my_app, :partitioned_cache, :primary, :command, :stop]`\n  * `[:my_app, :partitioned_cache, :command, :exception]`\n  * `[:my_app, :partitioned_cache, :primary, :command, :exception]`\n\nAs you may notice, the telemetry prefix by default for the cache is\n`[:my_app, :partitioned_cache]`. However, you could specify the\n`:telemetry_prefix` for the primary storage within the `:primary` options\n(if you want to override the default). See the\n[Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html)\nfor more information and examples.","ref":"Nebulex.Adapters.Partitioned.html#module-telemetry-events"},{"type":"module","title":"Adapter-specific telemetry events - Nebulex.Adapters.Partitioned","doc":"This adapter exposes following Telemetry events:\n\n  * `telemetry_prefix ++ [:bootstrap, :started]` - Dispatched by the adapter\n    when the bootstrap process is started.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node]\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap, :stopped]` - Dispatched by the adapter\n    when the bootstrap process is stopped.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node],\n        reason: term\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap, :exit]` - Dispatched by the adapter\n    when the bootstrap has received an exit signal.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node],\n        reason: term\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap, :joined]` - Dispatched by the adapter\n    when the bootstrap has joined the cache to the cluster.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node]\n      }\n      ```","ref":"Nebulex.Adapters.Partitioned.html#module-adapter-specific-telemetry-events"},{"type":"module","title":"Info API - Nebulex.Adapters.Partitioned","doc":"As explained above, the partitioned adapter depends on the configured primary\nstorage adapter. Therefore, the information the `info` command provides will\ndepend on the primary storage adapter. The Nebulex built-in adapters support\nthe recommended keys `:server`, `:memory`, and `:stats`. Additionally, the\npartitioned adapter supports:\n\n  * `:nodes_info` - A map with the info for each node.\n  * `:nodes` - A list with the cluster nodes.\n\nFor example, the info for `MyApp.PartitionedCache` may look like this:\n\n    iex> MyApp.PartitionedCache.info!()\n    %{\n      memory: %{total: nil, used: 344600},\n      server: %{\n        cache_module: MyApp.PartitionedCache,\n        cache_name: :partitioned_cache,\n        cache_adapter: Nebulex.Adapters.Partitioned,\n        cache_pid: #PID<0.1053.0>,\n        nbx_version: \"3.0.0\"\n      },\n      stats: %{\n        hits: 0,\n        misses: 0,\n        writes: 0,\n        evictions: 0,\n        expirations: 0,\n        deletions: 0,\n        updates: 0\n      },\n      nodes: [:\"node1@127.0.0.1\", ...],\n      nodes_info: %{\n        \"node1@127.0.0.1\": %{\n          memory: %{total: nil, used: 68920},\n          server: %{\n            cache_module: MyApp.PartitionedCache.Primary,\n            cache_name: MyApp.PartitionedCache.Primary,\n            cache_adapter: Nebulex.Adapters.Local,\n            cache_pid: #PID<23981.823.0>,\n            nbx_version: \"3.0.0\"\n          },\n          stats: %{\n            hits: 0,\n            misses: 0,\n            writes: 0,\n            evictions: 0,\n            expirations: 0,\n            deletions: 0,\n            updates: 0\n          }\n        },\n        ...\n      }\n    }","ref":"Nebulex.Adapters.Partitioned.html#module-info-api"},{"type":"module","title":"Extended API - Nebulex.Adapters.Partitioned","doc":"This adapter provides some additional convenience functions to the\n`Nebulex.Cache` API.\n\nRetrieving the primary storage or local cache module:\n\n    MyCache.__primary__()\n\nRetrieving the cluster nodes associated with the given cache `name`:\n\n    MyCache.nodes()\n\nGet a cluster node based on the given `key`:\n\n    MyCache.find_node(\"mykey\")\n\n    MyCache.find_node!(\"mykey\")\n\nJoining the cache to the cluster:\n\n    MyCache.join_cluster()\n\nLeaving the cluster (removes the cache from the cluster):\n\n    MyCache.leave_cluster()","ref":"Nebulex.Adapters.Partitioned.html#module-extended-api"},{"type":"module","title":"CAVEATS - Nebulex.Adapters.Partitioned","doc":"For `c:Nebulex.Cache.get_and_update/3` and `c:Nebulex.Cache.update/4`,\nthey both have a parameter that is the anonymous function, and it is compiled\ninto the module where it is created, which means it necessarily doesn't exists\non remote nodes. To ensure they work as expected, you must provide functions\nfrom modules existing in all nodes of the group.","ref":"Nebulex.Adapters.Partitioned.html#module-caveats"},{"type":"function","title":"Nebulex.Adapters.Partitioned.with_dynamic_cache/3","doc":"Helper function to use dynamic cache for internal primary cache storage\nwhen needed.","ref":"Nebulex.Adapters.Partitioned.html#with_dynamic_cache/3"},{"type":"module","title":"Nebulex.Distributed.Cluster","doc":"This module is used by cache adapters for distributed caching functionality.","ref":"Nebulex.Distributed.Cluster.html"},{"type":"function","title":"Nebulex.Distributed.Cluster.find_node/2","doc":"Finds a node in the ring.","ref":"Nebulex.Distributed.Cluster.html#find_node/2"},{"type":"function","title":"Nebulex.Distributed.Cluster.join/1","doc":"Joins the node where the cache `name`'s supervisor process is running to the\n`name`'s node group.","ref":"Nebulex.Distributed.Cluster.html#join/1"},{"type":"function","title":"Nebulex.Distributed.Cluster.leave/1","doc":"Makes the node where the cache `name`'s supervisor process is running, leave\nthe `name`'s node group.","ref":"Nebulex.Distributed.Cluster.html#leave/1"},{"type":"function","title":"Nebulex.Distributed.Cluster.monitor_scope/0","doc":"Wrapper for `:pg.monitor_scope/1`.","ref":"Nebulex.Distributed.Cluster.html#monitor_scope/0"},{"type":"function","title":"Nebulex.Distributed.Cluster.pg_child_spec/0","doc":"Returns the `:pg` child spec.","ref":"Nebulex.Distributed.Cluster.html#pg_child_spec/0"},{"type":"function","title":"Nebulex.Distributed.Cluster.pg_nodes/1","doc":"Returns the list of nodes joined to given `name`'s node group.","ref":"Nebulex.Distributed.Cluster.html#pg_nodes/1"},{"type":"function","title":"Nebulex.Distributed.Cluster.pg_scope/0","doc":"Returns the `:pg` scope.","ref":"Nebulex.Distributed.Cluster.html#pg_scope/0"},{"type":"function","title":"Nebulex.Distributed.Cluster.ring_nodes/1","doc":"Returns the list of nodes in the ring.","ref":"Nebulex.Distributed.Cluster.html#ring_nodes/1"},{"type":"module","title":"Nebulex.Distributed.RPC","doc":"RPC utilities.","ref":"Nebulex.Distributed.RPC.html"},{"type":"function","title":"Nebulex.Distributed.RPC.call/5","doc":"Evaluates `apply(mod, fun, args)` on node `node` and returns the corresponding\nevaluation result.\n\nA timeout, in milliseconds or `:infinity`, can be given with a default value\nof `5000`.","ref":"Nebulex.Distributed.RPC.html#call/5"},{"type":"function","title":"Example - Nebulex.Distributed.RPC.call/5","doc":"iex> Nebulex.Distributed.RPC.call(node(), Map, :new, [[]])\n    %{}","ref":"Nebulex.Distributed.RPC.html#call/5-example"},{"type":"type","title":"Nebulex.Distributed.RPC.mfa_call/0","doc":"Task callback","ref":"Nebulex.Distributed.RPC.html#t:mfa_call/0"},{"type":"function","title":"Nebulex.Distributed.RPC.multi_mfa_call/4","doc":"Similar to `multicall/7`, but it allows specifying the MFA per node.","ref":"Nebulex.Distributed.RPC.html#multi_mfa_call/4"},{"type":"function","title":"Example - Nebulex.Distributed.RPC.multi_mfa_call/4","doc":"iex> node = node()\n    iex> alias Nebulex.Distributed.RPC\n    iex> RPC.multi_mfa_call(%{node => {Map, :new, [[foo: :bar]]}})\n    {[{{:\"primary@127.0.0.1\", {Map, :new, [[foo: :bar]]}}, %{foo: :bar}}], []}\n    iex> RPC.multi_mfa_call(%{node => {Map, :new, [[foo: :bar]]}}, 1000)\n    {[{{:\"primary@127.0.0.1\", {Map, :new, [[foo: :bar]]}}, %{foo: :bar}}], []}\n    iex> RPC.multi_mfa_call(\n    ...>   %{node => {Map, :new, [[foo: :bar]]}},\n    ...>   1000,\n    ...>   {[], []}\n    ...> )\n    {[{{:\"primary@127.0.0.1\", {Map, :new, [[foo: :bar]]}}, %{foo: :bar}}], []}","ref":"Nebulex.Distributed.RPC.html#multi_mfa_call/4-example"},{"type":"function","title":"Nebulex.Distributed.RPC.multicall/7","doc":"In contrast to a regular single-node RPC, a multicall is an RPC that is sent\nconcurrently from one client to multiple servers. The function evaluates\n`apply(module, fun, args)` on the specified nodes and collects the answers.\nThen, evaluates the `reducer_fun` function on each answer.","ref":"Nebulex.Distributed.RPC.html#multicall/7"},{"type":"function","title":"Example - Nebulex.Distributed.RPC.multicall/7","doc":"iex> alias Nebulex.Distributed.RPC\n    iex> RPC.multicall([node()], Map, :new, [[foo: :bar]])\n    {[{:\"primary@127.0.0.1\", %{foo: :bar}}], []}\n    iex> RPC.multicall([node()], Map, :new, [[foo: :bar]], 1000)\n    {[{:\"primary@127.0.0.1\", %{foo: :bar}}], []}\n    iex> RPC.multicall([node()], Map, :new, [[foo: :bar]], 1000, {[], []})\n    {[{:\"primary@127.0.0.1\", %{foo: :bar}}], []}","ref":"Nebulex.Distributed.RPC.html#multicall/7-example"},{"type":"type","title":"Nebulex.Distributed.RPC.node_mfa_call/0","doc":"Group entry: node -> NFA call","ref":"Nebulex.Distributed.RPC.html#t:node_mfa_call/0"},{"type":"type","title":"Nebulex.Distributed.RPC.node_mfa_map/0","doc":"Node group","ref":"Nebulex.Distributed.RPC.html#t:node_mfa_map/0"},{"type":"type","title":"Nebulex.Distributed.RPC.reducer_acc/0","doc":"Reducer accumulator","ref":"Nebulex.Distributed.RPC.html#t:reducer_acc/0"},{"type":"type","title":"Nebulex.Distributed.RPC.reducer_fun/0","doc":"Reducer function spec","ref":"Nebulex.Distributed.RPC.html#t:reducer_fun/0"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.38.2"}}